//************************************************************************************************************************** 
//* Blue HttpMail Extension
//* Copyright (c) 2003-2004 Josh Harler
//* 
//* Blue - General Purpose C++ Library
//* Copyright (c) 2002-2004 Josh Harler
//* 
//* This software is provided 'as-is', without any express or implied warranty. In no event
//* will the authors be held liable for any damages arising from the use of this software.
//* 
//* Permission is granted to anyone to use this software for any purpose, including commercial
//* applications, and to alter it and redistribute it freely, subject to the following restrictions:
//* 
//* 	1. The origin of this software must not be misrepresented; you must not claim that you
//* 	wrote the original software. If you use this software in a product, an acknowledgment in the
//* 	product documentation would be appreciated but is not required.
//* 
//* 	2. Altered source versions must be plainly marked as such, and must not be misrepresented as
//* 	being the original software.
//* 
//* 	3. This notice may not be removed or altered from any source distribution.
//*
//*
//* file   Blue/Extension/HttpMail/Folder.h
//**

#ifndef __blue_ext_httpmail_Folder_h_included__
#define __blue_ext_httpmail_Folder_h_included__

// Public Headers ==========================================================================================================

// Extension headers
#include "Blue/Extension/HttpMail/Message.h"


// Public Defines/Enums/Typedefs/Etc. ======================================================================================

// Public Classes/Structs ==================================================================================================

namespace blue {
namespace ext {
namespace httpmail {

	/****
	 * Forward Declaration
	 */
	class Mailbox;


	/**
	 * \class Folder
	 * \brief Allows querying and manipulation of folders.
	 * \ingroup HttpMail
	 *
	 * This class offers a way for applications to query and manipulate
	 * folders on an HttpMail server.
	 *
	 * When using a folder for the first time, a call to refresh() is
	 * required so that the folder can contact the server and retrieve
	 * its list of folders and messages.
	 */
	class BLUE_EXPORT_HTTPMAIL Folder
	{
	public:
		// ===========================================================
		//  creation/destruction
		// ===========================================================

		/** Constructor. */
		Folder( Mailbox& mailbox, Folder* parent, String url );

		/** Destructor. */
		~Folder();


		// ===========================================================
		//  query
		// ===========================================================

		/** Gets the url used to access this folder. */
		String getUrl() const;

		/** Gets the name of the folder. */
		String getName() const;
		/** Gets the display name of the folder (what the user sees). */
		String getDisplayName() const;

		/** Gets the number of messages contained in this folder. */
		int getMessageCount() const;
		/** Gets the number of unread messages contained in this folder. */
		int getMessageCountUnread() const;

		/** Gets the number of folders contained in this folder. */
		int getFolderCount() const;
		
		/** Gets a folder using an index value. */
		Folder* getFolder( int index );
		/** overload */
		const Folder* getFolder( int index ) const;

		/** Gets a folder using the folder's name. */
		Folder* getFolder( String folderName );
		/** overload */
		const Folder* getFolder( String folderName ) const;

		/** Gets an array of all the folders contained in this folder. */
		Array<Folder*> getFolders();
		/** overload */
		const Array<Folder*> getFolders() const;

		/** Gets an email message using an index value. */
		Message* getMessage( int index );
		/** overload */
		const Message* getMessage( int index ) const;

		/** Gets an array of all the email messages contained in this folder. */
		Array<Message*> getMessages();
		/** overload */
		const Array<Message*> getMessages() const;

		/**
		 * Returns the parent folder of this folder.  For the root level
		 * folder, this returns 0.
		 */
		Folder* getParent();
		/** overload */
		const Folder* getParent() const;


		// ===========================================================
		//  manipulation
		// ===========================================================

		/**
		 * Sets the url used to access this folder.  This should only be
		 * called by the Mailbox class unless you're sure you know what
		 * you're doing.
		 */
		void setUrl( String url );
		/**
		 * Sets the name of the folder.  This should only be called by the
		 * Mailbox class unless you're sure you know what you're doing.
		 */
		void setName( String name );
		/**
		 * Sets the display name of the folder (what the user sees).
		 * This should only be called by the Mailbox class unless you're
		 * sure you know what you're doing.
		 */
		void setDisplay( String display );
		/**
		 * Sets the number of messages contained in this folder.  This
		 * should only be called by the Mailbox class unless you're sure
		 * you know what you're doing.
		 */
		void setMessageCount( int messageCnt );
		/**
		 * Sets the number of unread messages contained in this folder.
		 * This should only be called by the Mailbox class unless you're
		 * sure you know what you're doing.
		 */
		void setMessageCountUnread( int messageCntUnread );

		/**
		 * Creates a new folder within this folder.
		 * \returns the index used to access the new folder.
		 */
		int createFolder( String folderName );
		/** Removes a folder from this folder. */
		void removeFolder( int index );
		/** overload */
		void removeFolder( String folderName );

		/**
		 * Moves a message from this folder to the given folder.
		 */
		void moveMessage( int index, Folder* to );
		/** overload */
		void moveMessage( Message* message, Folder* to );

		/**
		 * Copies a message from this folder to the given folder.
		 */
		void copyMessage( int index, Folder* to );
		/** overload */
		void copyMessage( Message* message, Folder* to );

		/**
		 * Deletes the given message.
		 */
		void removeMessage( int index );
		/** overload */
		void removeMessage( Message* message );

		/**
		 * Marks the message as read or unread on the server.
		 */
		void setMessageRead( int index, bool read );
		/** overload */
		void setMessageRead( Message* message, bool read );

		/**
		 * Refreshes the information about the folders and messages
		 * contained within this folder.
		 */
		void refresh();


	protected:

	private:
		/**
		 * Copy constructor. Private because Folders should not be
		 * manipulated by more than one instance.
		 */
		Folder( const Folder& );
		/** Private assignment operator. See copy constructor documentation. */
		const Folder& operator=( const Folder& );

		Mailbox& m_mailbox;
		Folder*  m_parent;
		String   m_url;
		String   m_name;
		String   m_display;
		int      m_msgCount;
		int32_t  m_msgCountUnread;
		
		Array<Folder*> m_folders;
		Array<Message*> m_messages;
	};



	/**
	 * \class FolderException
	 * \brief Thrown when an exception occurs when manipulating a Folder.
	 * \ingroup HttpMailExceptions
	 */
	class BLUE_EXPORT_HTTPMAIL FolderException :public HttpMailException
	{
	public:
		FolderException( String desc )
			:HttpMailException(desc) {}

		virtual String getException() const {
			return ("FolderException");
		}
	};


}}}	// namespaces


// Public External Variables ===============================================================================================

// Public Function Prototypes ==============================================================================================

// Public Inline Functions =================================================================================================

#endif // include guard
